@@ -8,19 +8,22 @@ from typing import (
     Dict,
     Union,
     Sequence,
+    Any,
 )
 from .utility import KeylessCache, Cache, CacheConfig, Requests, InsensitiveEnum
-from .utility.exceptions import *
+from .exceptions import *
 from .models import *
 import asyncio
 import httpx
 
 from .api_types.v1 import *
+from .api_types.v1 import _APIMap
 
 if TYPE_CHECKING:
     from .client import PRC
 
 R = TypeVar("R")
+M = TypeVar("M")
 
 
 class ServerCache:
@@ -71,7 +74,7 @@ class Server:
         self,
         client: "PRC",
         server_key: str,
-        ephemeral_ttl: int = 5,
+        ephemeral_ttl: int = 3,
         cache: ServerCache = ServerCache(),
         requests: Optional[Requests] = None,
         ignore_global_key: bool = False,
@@ -87,8 +90,24 @@ class Server:
 
         self._global_key = client._global_key
         self._server_key = server_key
-        self._requests = requests or self._refresh_requests()
         self._ignore_global_key = ignore_global_key
+        self._requests: Requests[
+            Literal[
+                "/",
+                "/players",
+                "/queue",
+                "/bans",
+                "/vehicles",
+                "/staff",
+                "/joinlogs",
+                "/killlogs",
+                "/commandlogs",
+                "/modcalls",
+                "/command",
+            ]
+        ] = (
+            requests or self._refresh_requests()
+        )
 
         self.logs = ServerLogs(self)
         self.commands = ServerCommands(self)
@@ -116,6 +135,18 @@ class Server:
             else None
         )
 
+    def is_online(self):
+        """Whether the server is online (i.e. has online players). Server status or players must be fetched separately."""
+        return self.player_count > 0 if self.player_count else None
+
+    def is_full(self, include_reserved: bool = False):
+        """Whether the server player count has reached the max player limit. Excludes owner-reserved spot by default (`max_players - 1`), set `include_reserved=True` to include. Server status must be fetched separately."""
+        return (
+            (self.player_count >= self.max_players - (0 if include_reserved else 1))
+            if self.player_count and self.max_players
+            else None
+        )
+
     def _refresh_requests(self):
         global_key = self._global_key
         headers = {"Server-Key": self._server_key}
@@ -128,6 +159,11 @@ class Server:
         )
         return self._requests
 
+    def _parse_api_map(self, map: _APIMap[M]) -> Dict[str, M]:
+        if not isinstance(map, Dict):
+            return {}
+        return map
+
     def _get_player(self, id: Optional[int] = None, name: Optional[str] = None):
         for _, player in self._server_cache.players.items():
             if id and player.id == id:
@@ -135,47 +171,60 @@ class Server:
             if name and player.name == name:
                 return player
 
-    def _handle_error_code(self, error_code: Optional[int] = None):
-        if error_code is None:
-            raise PRCException("An unknown error has occured.")
+    def _raise_error_code(self, response: Any):
+        if not isinstance(response, Dict):
+            raise PRCException("A malformed response was received.")
 
-        errors: List[Callable[..., APIException]] = [
+        error_code = response.get("code")
+        if error_code is None:
+            raise PRCException("No error was received.")
+
+        exceptions: List[Callable[..., APIException]] = [
             UnknownError,
             CommunicationError,
             InternalError,
-            MissingServerKey,
-            InvalidServerKeyFormat,
             InvalidServerKey,
             InvalidGlobalKey,
             BannedServerKey,
             InvalidCommand,
             ServerOffline,
-            RateLimit,
+            RateLimited,
             RestrictedCommand,
             ProhibitedMessage,
             RestrictedResource,
             OutOfDateModule,
         ]
 
-        for error in errors:
-            error = error()
-            if error_code == error.error_code:
+        for _exception in exceptions:
+            exception = _exception()
+            if error_code == exception.code:
                 invalid_key = None
-                if isinstance(error, InvalidGlobalKey):
-                    invalid_key = self._requests._default_headers.get("Authorization")
-                elif isinstance(error, (InvalidServerKey, BannedServerKey)):
-                    invalid_key = self._requests._default_headers.get("Server-Key")
+                if isinstance(exception, InvalidGlobalKey):
+                    invalid_key = self._global_key
+                elif isinstance(exception, (InvalidServerKey, BannedServerKey)):
+                    invalid_key = self._server_key
 
                 if invalid_key:
                     self._requests._invalid_keys.add(invalid_key)
 
-                raise error
+                if isinstance(exception, RateLimited):
+                    exception = RateLimited(
+                        response.get("bucket"), response.get("retry_after")
+                    )
 
-        raise APIException(error_code, "An unknown API error has occured.")
+                if isinstance(exception, (CommunicationError, ServerOffline)):
+                    exception = _exception(command_id=response.get("commandId"))
+
+                raise exception
+
+        raise APIException(
+            error_code,
+            f"An unknown API error has occured: {response.get('message') or '...'}",
+        )
 
     def _handle(self, response: httpx.Response, return_type: Type[R]) -> R:
         if not response.is_success:
-            self._handle_error_code((response.json() or {}).get("code"))
+            self._raise_error_code(response.json())
         return response.json()
 
     @_refresh_server
@@ -183,7 +232,8 @@ class Server:
     async def get_status(self):
         """Get the current server status."""
         return ServerStatus(
-            self, data=self._handle(await self._requests.get("/"), ServerStatusResponse)
+            self,
+            data=self._handle(await self._requests.get("/"), v1_ServerStatusResponse),
         )
 
     @_refresh_server
@@ -193,20 +243,21 @@ class Server:
         players = [
             ServerPlayer(self, data=p)
             for p in self._handle(
-                await self._requests.get("/players"), List[ServerPlayerResponse]
+                await self._requests.get("/players"), v1_ServerPlayersResponse
             )
         ]
         self.player_count = len(players)
         self.staff_count = len([p for p in players if p.is_staff()])
         return players
 
+    @_refresh_server
     @_ephemeral
     async def get_queue(self):
         """Get all players in the server join queue."""
         players = [
             QueuedPlayer(self, id=p, index=i)
             for i, p in enumerate(
-                self._handle(await self._requests.get("/queue"), List[int])
+                self._handle(await self._requests.get("/queue"), v1_ServerQueueResponse)
             )
         ]
         self.queue_count = len(players)
@@ -217,9 +268,9 @@ class Server:
     async def get_bans(self):
         """Get all banned players."""
         return [
-            Player(self._client, data=p)
-            for p in (
-                self._handle(await self._requests.get("/bans"), ServerBanResponse) or {}
+            Player(self._client, data=p, _skip_cache=True)
+            for p in self._parse_api_map(
+                self._handle(await self._requests.get("/bans"), v1_ServerBanResponse)
             ).items()
         ]
 
@@ -230,7 +281,7 @@ class Server:
         return [
             Vehicle(self, data=v)
             for v in self._handle(
-                await self._requests.get("/vehicles"), List[ServerVehicleResponse]
+                await self._requests.get("/vehicles"), v1_ServerVehiclesResponse
             )
         ]
 
@@ -240,7 +291,9 @@ class Server:
         """Get all server staff members excluding server owner. ⚠️ *(This endpoint is deprecated, use at your own risk)*"""
         staff = ServerStaff(
             self,
-            data=self._handle(await self._requests.get("/staff"), ServerStaffResponse),
+            data=self._handle(
+                await self._requests.get("/staff"), v1_ServerStaffResponse
+            ),
         )
         self.total_staff_count = staff.count()
         return staff
@@ -273,7 +326,7 @@ class ServerLogs(ServerModule):
         [
             AccessEntry(self._server, data=e)
             for e in self._handle(
-                await self._requests.get("/joinlogs"), List[ServerJoinLogResponse]
+                await self._requests.get("/joinlogs"), v1_ServerJoinLogsResponse
             )
         ]
         return self._server_cache.access_logs.items()
@@ -285,7 +338,7 @@ class ServerLogs(ServerModule):
         return [
             KillEntry(self._server, data=e)
             for e in self._handle(
-                await self._requests.get("/killlogs"), List[ServerKillLogResponse]
+                await self._requests.get("/killlogs"), v1_ServerKillLogsResponse
             )
         ]
 
@@ -296,7 +349,7 @@ class ServerLogs(ServerModule):
         return [
             CommandEntry(self._server, data=e)
             for e in self._handle(
-                await self._requests.get("/commandlogs"), List[ServerCommandLogResponse]
+                await self._requests.get("/commandlogs"), v1_ServerCommandLogsResponse
             )
         ]
 
@@ -307,13 +360,13 @@ class ServerLogs(ServerModule):
         return [
             ModCallEntry(self._server, data=e)
             for e in self._handle(
-                await self._requests.get("/modcalls"), List[ServerModCallResponse]
+                await self._requests.get("/modcalls"), v1_ServerModCallsResponse
             )
         ]
 
 
-CommandTargetPlayerName = str
-CommandTargetPlayerId = int
+CommandTargetPlayerName = Union[str, Player]
+CommandTargetPlayerId = Union[int, Player]
 CommandTargetPlayerNameOrId = Union[CommandTargetPlayerName, CommandTargetPlayerId]
 
 
@@ -324,10 +377,10 @@ class ServerCommands(ServerModule):
         super().__init__(server)
 
     async def _raw(self, command: str):
-        """Run a raw string command as the remote player in the server."""
+        """Send a raw command string to the remote command execution API."""
         return self._handle(
             await self._requests.post("/command", json={"command": command}),
-            Dict,
+            v1_ServerCommandExecutionResponse,
         )
 
     async def run(
@@ -336,12 +389,21 @@ class ServerCommands(ServerModule):
         targets: Optional[Sequence[CommandTargetPlayerNameOrId]] = None,
         args: Optional[List[CommandArg]] = None,
         text: Optional[str] = None,
+        _max_retries: int = 3,
+        _prefer_player_id: bool = False,
     ):
         """Run any command as the remote player in the server."""
         command = f":{name} "
 
+        def parse_target(target: CommandTargetPlayerNameOrId):
+            if isinstance(target, Player):
+                if _prefer_player_id:
+                    return str(target.id)
+                return str(target.name)
+            return str(target)
+
         if targets:
-            command += ",".join([str(t) for t in targets]) + " "
+            command += ",".join([parse_target(t) for t in targets]) + " "
 
         if args:
             command += (
@@ -357,7 +419,19 @@ class ServerCommands(ServerModule):
         if text:
             command += text
 
-        await self._raw(command.strip())
+        message = "..."
+        success = False
+        retry = 0
+
+        while success == False and retry < _max_retries:
+            message = (await self._raw(command.strip())).get("message")
+            success = message == "Success"
+            retry += 1
+
+        if not success:
+            raise PRCException(
+                f"An unknown command execution error has occured: {message}"
+            )
 
     async def kill(self, targets: List[CommandTargetPlayerName]):
         """Kill players in the server."""
@@ -405,41 +479,41 @@ class ServerCommands(ServerModule):
 
     async def ban(self, targets: List[CommandTargetPlayerNameOrId]):
         """Ban players from the server."""
-        await self.run("ban", targets=targets)
+        await self.run("ban", targets=targets, _prefer_player_id=True)
 
     async def unban(self, targets: List[CommandTargetPlayerNameOrId]):
         """Unban players from the server."""
-        await self.run("unban", targets=targets)
+        await self.run("unban", targets=targets, _prefer_player_id=True)
 
     async def grant_helper(self, targets: List[CommandTargetPlayerNameOrId]):
         """Grant helper permissions to players in the server."""
-        await self.run("helper", targets=targets)
+        await self.run("helper", targets=targets, _prefer_player_id=True)
 
     async def revoke_helper(self, targets: List[CommandTargetPlayerNameOrId]):
         """Revoke helper permissions to players in the server."""
-        await self.run("unhelper", targets=targets)
+        await self.run("unhelper", targets=targets, _prefer_player_id=True)
 
     async def grant_mod(self, targets: List[CommandTargetPlayerNameOrId]):
         """Grant moderator permissions to players in the server."""
-        await self.run("mod", targets=targets)
+        await self.run("mod", targets=targets, _prefer_player_id=True)
 
     async def revoke_mod(self, targets: List[CommandTargetPlayerNameOrId]):
         """Revoke moderator permissions from players in the server."""
-        await self.run("unmod", targets=targets)
+        await self.run("unmod", targets=targets, _prefer_player_id=True)
 
     async def grant_admin(self, targets: List[CommandTargetPlayerNameOrId]):
         """Grant admin permissions to players in the server."""
-        await self.run("admin", targets=targets)
+        await self.run("admin", targets=targets, _prefer_player_id=True)
 
     async def revoke_admin(self, targets: List[CommandTargetPlayerNameOrId]):
         """Revoke admin permissions from players in the server."""
-        await self.run("unadmin", targets=targets)
+        await self.run("unadmin", targets=targets, _prefer_player_id=True)
 
     async def send_hint(self, text: str):
         """Send a temporary message to the server (undismissable banner)."""
         await self.run("h", text=text)
 
-    async def send_message(self, text: str):
+    async def send_announcement(self, text: str):
         """Send an announcement message to the server (dismissable popup)."""
         await self.run("m", text=text)
 
@@ -447,7 +521,7 @@ class ServerCommands(ServerModule):
         """Send a private message to players in the server (dismissable popup)."""
         await self.run("pm", targets=targets, text=text)
 
-    async def log(self, text: str):
+    async def send_log(self, text: str):
         """Emit a custom string that will be saved in command logs and sent to configured command usage webhooks (if any), mostly for integrating with other applications. Uses the `:log` command."""
         await self.run("log", text=text)
 
