@@ -1,9 +1,9 @@
 """
 
-Internal prc.api utilities and exceptions.
+Internal prc.api utilities.
 
 """
 
-from .enum import InsensitiveEnum
+from .enum import InsensitiveEnum, DisplayNameEnum
 from .cache import KeylessCache, Cache, CacheConfig
 from .requests import Requests
